#include "mwv207_backlight.h"
/*
* SPDX-License-Identifier: GPL
*
* Copyright (c) 2020 ChangSha JingJiaMicro Electronics Co., Ltd.
* All rights reserved.
*
* Author:
*      shanjinkui <shanjinkui@jingjiamicro.com>
*
* The software and information contained herein is proprietary and
* confidential to JingJiaMicro Electronics. This software can only be
* used by JingJiaMicro Electronics Corporation. Any use, reproduction,
* or disclosure without the written permission of JingJiaMicro
* Electronics Corporation is strictly prohibited.
*/
#include "mwv207_vbios.h"

static int bl_gpio_config(mwv207_bl_hw *bl_hw) {
        mutex_lock(bl_hw->mutex);
        if (bl_hw->pnl_bl_en == 0x10) {
                jdev_modify(bl_hw->jdev, MWV207_GPIO_MULTIPLEX16_23, 0x3, 0);
                jdev_modify(bl_hw->jdev, MWV207_GPIO_MULTIPLEX8_15, 0x3 << 16,
                            0x1 << 16);
                jdev_modify(bl_hw->jdev, MWV207_GPIO_DIR_L,
                            0x1 << 0xC, 0);
        } else if (bl_hw->pnl_bl_en == 0x12) {
                jdev_modify(bl_hw->jdev, MWV207_GPIO_MULTIPLEX16_23, 0x3 << 8,
                            0);
                jdev_modify(bl_hw->jdev, MWV207_GPIO_MULTIPLEX8_15, 0x3 << 20,
                            0x1 << 20);
                jdev_modify(bl_hw->jdev, MWV207_GPIO_DIR_L,
                            0x1 << 0xD, 0);
        }
        mutex_unlock(bl_hw->mutex);

        DRM_DEBUG_DRIVER("jmgpu: backlight gpio config, pnl_bl_en:%u "
                         "pnl_bl_pwm:%u pwm_idx:%u",
                         bl_hw->pnl_bl_en, bl_hw->pnl_bl_pwm, bl_hw->pwm_idx);
        return 0;
}

static bool bl_en_state(mwv207_bl_hw *bl_hw) {
        bool ret = false;

        mutex_lock(bl_hw->mutex);
        if (jdev_read(bl_hw->jdev, MWV207_GPIO_DIR_L) &
            (0x1 << bl_hw->pnl_bl_en)) {
                ret = true;
        } else {
                ret = false;
        }

        return ret;
}

static int bl_en_switch(mwv207_bl_hw *bl_hw, int on) {
        bool current_state = false;

        current_state = bl_hw->ops->bl_en_state(bl_hw);
        if (current_state == on) {
                jdev_modify(bl_hw->jdev, 0x9AD080, 0x1 << bl_hw->pwm_idx,
                            on ? 0x1 << bl_hw->pwm_idx : 0);
                return 0;
        }

        mutex_lock(bl_hw->mutex);
        if (on) {
                jdev_modify(bl_hw->jdev, MWV207_GPIO_MULTIPLEX8_15,
                            0x3 << (12 + bl_hw->pwm_idx * 4),
                            0x1 << (12 + bl_hw->pwm_idx * 4));
                jdev_modify(bl_hw->jdev, MWV207_GPIO_DIR_L,
                            0x1 << bl_hw->pnl_bl_pwm, 0x0 << bl_hw->pnl_bl_pwm);
                jdev_modify(bl_hw->jdev, 0x9AD080, 0x1 << bl_hw->pwm_idx,
                            on ? 0x1 << bl_hw->pwm_idx : 0);
                jdev_modify(bl_hw->jdev, MWV207_GPIO_OUT_L,
                            0x1 << bl_hw->pnl_bl_en,
                            on ? 0x1 << bl_hw->pnl_bl_en : 0);
        } else {
                jdev_modify(bl_hw->jdev, MWV207_GPIO_OUT_L,
                            0x1 << bl_hw->pnl_bl_en,
                            on ? 0x1 << bl_hw->pnl_bl_en : 0);
                usleep_range(1000, 2000);
                jdev_modify(bl_hw->jdev, MWV207_GPIO_DIR_L,
                            0x1 << bl_hw->pnl_bl_pwm, 0x0 << bl_hw->pnl_bl_pwm);
                jdev_modify(bl_hw->jdev, 0x9AD080, 0x1 << bl_hw->pwm_idx,
                            on ? 0x1 << bl_hw->pwm_idx : 0);
                jdev_modify(bl_hw->jdev, MWV207_GPIO_MULTIPLEX8_15,
                            0x3 << (12 + bl_hw->pwm_idx * 4),
                            0x0 << (12 + bl_hw->pwm_idx * 4));
        }

        mutex_unlock(bl_hw->mutex);
        return 0;
}

static int bl_update_pwm(mwv207_bl_hw *bl_hw, u32 highLevelTime, u32 totalTime,
                         u32 forwareTime) {
        jdev_write(bl_hw->jdev, MWV207_PWM_HIGHCYCLENUM(bl_hw->pwm_idx),
                   highLevelTime);
        jdev_write(bl_hw->jdev, MWV207_PWM_TOTALCYCLENUM(bl_hw->pwm_idx),
                   totalTime);
        jdev_write(bl_hw->jdev, MWV207_PWM_FORWARDTIME(bl_hw->pwm_idx),
                   forwareTime);

        bl_hw->ops->bl_en_switch(bl_hw, 0x1);
        return 0;
}

static const mwv207_bl_hw_ops bl_hw_ops = {.bl_gpio_config = bl_gpio_config,
                                           .bl_update_pwm = bl_update_pwm,
                                           .bl_en_switch = bl_en_switch,
                                           .bl_en_state = bl_en_state};

static int bl_update(mwv207_backlight *mwv207_bl, int brightness) {
        mwv207_bl_hw *bl_hw = mwv207_bl->hw;
        u32 high_time;

        high_time =
            brightness * mwv207_bl->brightness_step + mwv207_bl->min_brightness;
        high_time *= MWV207_BL_STEP;
        if (high_time > 0xE7EF0)
                high_time = 0xE7EF0;

        return bl_hw->ops->bl_update_pwm(bl_hw, high_time, 0xE7EF0,
                                         0x0);
}

static int bl_update_status(struct backlight_device *bl) {
        int brightness = bl->props.brightness;
        mwv207_backlight *mwv207_bl = bl_get_data(bl);
        mwv207_bl_hw *bl_hw = mwv207_bl->hw;

        bl_hw->ops->bl_gpio_config(bl_hw);

        if (bl->props.power != FB_BLANK_UNBLANK ||
            bl->props.fb_blank != FB_BLANK_UNBLANK ||
            bl->props.state & BL_CORE_FBBLANK) {
                bl_hw->ops->bl_en_switch(bl_hw, 0x1);
                return 0;
        }

        brightness *= mwv207_bl->max_brightness;
        brightness /= mwv207_bl->max_brightness;
        if (brightness > mwv207_bl->max_level)
                brightness = mwv207_bl->max_level;

        mwv207_bl->current_brightness = brightness;

        bl_update(mwv207_bl, brightness);
        return 0;
}

static int bl_get_brightness(struct backlight_device *bl) {
        mwv207_backlight *mwv207_bl = bl_get_data(bl);

        return mwv207_bl->current_brightness;
}

static const struct backlight_ops mwv207_bl_ops = {
    .update_status = bl_update_status,
    .get_brightness = bl_get_brightness,
};

static void devm_backlight_device_release(struct device *dev, void *res) {
        struct backlight_device *backlight = *(struct backlight_device **)res;
        backlight_device_unregister(backlight);
}

static int devm_backlight_device_match(struct device *dev, void *res,
                                       void *data) {
        struct backlight_device **r = res;
        return *r == data;
}

struct backlight_device *devm_backlight_device_register(
    struct device *dev, const char *name, struct device *parent, void *devdata,
    const struct backlight_ops *ops, const struct backlight_properties *props) {
        struct backlight_device **ptr, *backlight;

        ptr = devres_alloc(devm_backlight_device_release, sizeof(*ptr),
                           GFP_KERNEL);

        if (!ptr)
                return ERR_PTR(-ENOMEM);

        backlight =
            backlight_device_register(name, parent, devdata, ops, props);
        if (!IS_ERR(backlight)) {
                *ptr = backlight;
                devres_add(dev, ptr);
        } else {
                devres_free(ptr);
        }

        return backlight;
}

void devm_backlight_device_unregister(struct device *dev,
                                      struct backlight_device *db) {
        int rc;

        rc = devres_release(dev, devm_backlight_device_release,
                            devm_backlight_device_match, db);
        WARN_ON(rc);
}

static bool bl_get_cfg(struct mwv207_device *jdev, u32 key, u32 *val) {
        const struct mwv207_vdat *vdat;
        bool ret = false;

        vdat = mwv207_vbios_vdat(jdev, key);
        if (!vdat) {
                goto parse_fail;
        }

        *val = *(u32 *)vdat->dat;
        ret = true;

parse_fail:
        return ret;
}

static void bl_parse_cfg(mwv207_backlight *jbacklight,
                         struct mwv207_device *jdev, u32 key_base) {
        char *cfg_name[3] = {"max_level", "max_brigtness", "min_brightness"};
        u32 cfg_val[3] = {20, 100, 1};
        u32 bl_step;
        bool found;
        u32 val;
        int i;

        for (i = 0; i < 3; i++) {
                found = bl_get_cfg(jdev, key_base + i, &val);
                if (!found) {
                        DRM_INFO(
                            "backlight read cfg %s not exist, use default: %u",
                            cfg_name[i], cfg_val[i]);
                        val = cfg_val[i];
                } else if (val > 100 || val < 1) {
                        DRM_INFO(
                            "backlight cfg %s value invalid, use default: %u",
                            cfg_name[i], cfg_val[i]);
                        val = cfg_val[i];
                }
                cfg_val[i] = val;
        }

        val = cfg_val[1] - cfg_val[2];
        bl_step = val % cfg_val[0] ? val / cfg_val[0] + 1 : val / cfg_val[0];

        jbacklight->max_level = cfg_val[0];
        jbacklight->max_brightness = cfg_val[1];
        jbacklight->min_brightness = cfg_val[2];
        jbacklight->brightness_step = bl_step;

        DRM_DEBUG_DRIVER("backlight parse cfg min_br:%d max_br:%d br_step:%d",
                         jbacklight->min_brightness, jbacklight->max_brightness,
                         jbacklight->brightness_step);
}

static void bl_hw_init(struct drm_device *ddev, mwv207_backlight *jbacklight,
                       struct mwv207_device *jdev, u32 chan) {
        mwv207_bl_hw *bl_hw =
            devm_kzalloc(ddev->dev, sizeof(mwv207_bl_hw), GFP_KERNEL);
        jbacklight->hw = bl_hw;
        bl_hw->jdev = jdev;
        bl_hw->mutex = &jdev->gpio_lock;

        if (chan == 1) {
                bl_hw->pnl_bl_en = 0x10;
                bl_hw->pnl_bl_pwm = 0xC;
                bl_hw->pwm_idx = 1;
        }

        bl_hw->ops = &bl_hw_ops;
}

int bl_init(struct drm_device *ddev, struct backlight_device **bl_dev,
            struct mwv207_device *jdev, u32 key_idx, u32 conn_type) {
        struct backlight_properties bl_props;
        mwv207_backlight *jbacklight;
        u32 bl_chan, key;
        char name[32];

        key = key_idx * 20 + 3;

        if (!bl_get_cfg(jdev, key, &bl_chan)) {
                DRM_DEBUG_DRIVER("jmgpu: backlight bl_chan not set in cfg");
                DRM_DEBUG_DRIVER("jmgpu: backlight disabled");
                return 0;
        } else if (bl_chan > 2) {
                DRM_DEBUG_DRIVER("jmgpu: backlight bl_chan:%u is invalid",
                                 bl_chan);
                return 0;
        } else if (bl_chan == 0) {
                DRM_DEBUG_DRIVER("jmgpu: backlight disabled, key_idx:%d",
                                 key_idx);
                return 0;
        }

        jbacklight =
            devm_kzalloc(ddev->dev, sizeof(mwv207_backlight), GFP_KERNEL);
        jbacklight->enabled = true;

        sprintf(name, "jmgpu-backlight-%d", bl_chan);

        bl_parse_cfg(jbacklight, jdev, key + 1);

        bl_hw_init(ddev, jbacklight, jdev, bl_chan);

        memset(&bl_props, 0, sizeof(struct backlight_properties));
        bl_props.type = BACKLIGHT_RAW;
        bl_props.max_brightness = jbacklight->max_level;
        bl_props.brightness = jbacklight->max_level / 2;

        jbacklight->backlight_dev = devm_backlight_device_register(
            ddev->dev, name, ddev->dev, jbacklight, &mwv207_bl_ops, &bl_props);

        *bl_dev = jbacklight->backlight_dev;
        DRM_INFO("jmgpu: backlight bl_chan:%u enabled, bind conn_type: %d",
                 bl_chan, conn_type);
        return 0;
}

void bl_fini(struct backlight_device *bl_dev) {
        if (!bl_dev)
                return;
        devm_backlight_device_unregister(&bl_dev->dev, bl_dev);
}
