#include "../mwv207_drv.h"
/*
* SPDX-License-Identifier: GPL
*
* Copyright (c) 2020 ChangSha JingJiaMicro Electronics Co., Ltd.
* All rights reserved.
*
* Author:
*      shanjinkui <shanjinkui@jingjiamicro.com>
*
* The software and information contained herein is proprietary and
* confidential to JingJiaMicro Electronics. This software can only be
* used by JingJiaMicro Electronics Corporation. Any use, reproduction,
* or disclosure without the written permission of JingJiaMicro
* Electronics Corporation is strictly prohibited.
*/
#include <linux/backlight.h>

#define MWV207_BL_PWM_FREQ (0x389FD980 / 0xE7EF0)
#define MWV207_BL_STEP (0xE7EF0 / 0x64)

#define MWV207_GPIO_BASE_ADDR (0x009aa000)
#define MWV207_GPIO_MULTIPLEX_BASE_ADDR (0x009b0910)

#define MWV207_GPIO_MULTIPLEX8_15 (MWV207_GPIO_MULTIPLEX_BASE_ADDR + 0x4)
#define MWV207_GPIO_MULTIPLEX16_23 (MWV207_GPIO_MULTIPLEX_BASE_ADDR + 0x8)
#define MWV207_GPIO_OUT_L (MWV207_GPIO_BASE_ADDR + 0x4)
#define MWV207_GPIO_DIR_L (MWV207_GPIO_BASE_ADDR + 0x8)

#define MWV207_PWM_BASE_ADDR_C(_chan)                                          \
        (0x9AD000 + 0x20 * (_chan))
#define MWV207_PWM_TOTALCYCLENUM(_chan) (MWV207_PWM_BASE_ADDR_C(_chan) + 0x0)
#define MWV207_PWM_HIGHCYCLENUM(_chan) (MWV207_PWM_BASE_ADDR_C(_chan) + 0x4)
#define MWV207_PWM_FORWARDTIME(_chan) (MWV207_PWM_BASE_ADDR_C(_chan) + 0x8)

typedef struct tag_mwv207_bl_hw mwv207_bl_hw;

typedef struct tag_mwv207_hw_bl_ops {
        int (*bl_gpio_config)(mwv207_bl_hw *bl_hw);
        int (*bl_en_switch)(mwv207_bl_hw *bl_hw, int on);
        int (*bl_update_pwm)(mwv207_bl_hw *bl_hw, u32 highLevelTime,
                             u32 totalTime, u32 forwareTime);
        bool (*bl_en_state)(mwv207_bl_hw *bl_hw);
} mwv207_bl_hw_ops;

typedef struct tag_mwv207_bl_hw {
        struct mwv207_device *jdev;
        const mwv207_bl_hw_ops *ops;
        struct mutex *mutex;
        u32 pnl_bl_en;
        u32 pnl_bl_pwm;
        u32 pwm_idx;
} mwv207_bl_hw;

typedef struct tag_mwv207_backlight {
        struct backlight_device *backlight_dev;
        mwv207_bl_hw *hw;
        u32 current_brightness;
        u32 min_brightness;
        u32 max_brightness;
        u32 brightness_step;
        u32 max_level;
        bool enabled;
} mwv207_backlight;

int bl_init(struct drm_device *ddev, struct backlight_device **bl_dev,
            struct mwv207_device *jdev, u32 key_idx, u32 conn_type);
void bl_fini(struct backlight_device *bl_dev);
