/*
* SPDX-License-Identifier: GPL
*
* Copyright (c) 2020 ChangSha JingJiaMicro Electronics Co., Ltd.
* All rights reserved.
*
* Author:
*      shanjinkui <shanjinkui@jingjiamicro.com>
*
* The software and information contained herein is proprietary and
* confidential to JingJiaMicro Electronics. This software can only be
* used by JingJiaMicro Electronics Corporation. Any use, reproduction,
* or disclosure without the written permission of JingJiaMicro
* Electronics Corporation is strictly prohibited.
*/

#ifndef MWV207_DP_H
#define MWV207_DP_H

#include <linux/types.h>

struct drm_display_info;
struct drm_display_mode;
struct drm_dp_aux;
struct mwv207_dp_link;

struct mwv207_dp_link_caps {
	bool enhanced_framing;

	bool tps3_supported;

	bool fast_training;

	bool channel_coding;

	bool alternate_scrambler_reset;
};

void mwv207_dp_link_caps_copy(struct mwv207_dp_link_caps *dest,
			   const struct mwv207_dp_link_caps *src);

struct mwv207_dp_link_ops {
	int (*apply_training)(struct mwv207_dp_link *link);

	int (*configure)(struct mwv207_dp_link *link);
};

#define DP_TRAIN_VOLTAGE_SWING_LEVEL(x) ((x) << 0)
#define DP_TRAIN_PRE_EMPHASIS_LEVEL(x) ((x) << 3)
#define DP_LANE_POST_CURSOR(i, x) (((x) & 0x3) << (((i) & 1) << 2))

struct mwv207_dp_link_train_set {
	unsigned int voltage_swing[4];
	unsigned int pre_emphasis[4];
	unsigned int post_cursor[4];
};

struct mwv207_dp_link_train {
	struct mwv207_dp_link_train_set request;
	struct mwv207_dp_link_train_set adjust;

	unsigned int pattern;

	bool clock_recovered;
	bool channel_equalized;
};

struct mwv207_dp_link {
	unsigned char revision;
	unsigned int max_rate;
	unsigned int max_lanes;

	struct mwv207_dp_link_caps caps;

	struct {
		unsigned int cr;
		unsigned int ce;
	} aux_rd_interval;

	unsigned char edp;

	unsigned int rate;
	unsigned int lanes;

	unsigned long rates[DP_MAX_SUPPORTED_RATES];
	unsigned int num_rates;

	const struct mwv207_dp_link_ops *ops;

	struct drm_dp_aux *aux;

	struct mwv207_dp_link_train train;
};

int mwv207_dp_link_add_rate(struct mwv207_dp_link *link, unsigned long rate);
int mwv207_dp_link_remove_rate(struct mwv207_dp_link *link, unsigned long rate);
void mwv207_dp_link_update_rates(struct mwv207_dp_link *link);

int mwv207_dp_link_probe(struct drm_dp_aux *aux, struct mwv207_dp_link *link);
int mwv207_dp_link_power_up(struct drm_dp_aux *aux, struct mwv207_dp_link *link);
int mwv207_dp_link_power_down(struct drm_dp_aux *aux, struct mwv207_dp_link *link);
int mwv207_dp_link_configure(struct drm_dp_aux *aux, struct mwv207_dp_link *link);
int mwv207_dp_link_choose(struct mwv207_dp_link *link,
		       const struct drm_display_mode *mode,
		       const struct drm_display_info *info);

void mwv207_dp_link_train_init(struct mwv207_dp_link_train *train);
int mwv207_dp_link_train(struct mwv207_dp_link *link);

#endif
