/*
* SPDX-License-Identifier: GPL
*
* Copyright (c) 2020 ChangSha JingJiaMicro Electronics Co., Ltd.
* All rights reserved.
*
* Author:
*      shanjinkui <shanjinkui@jingjiamicro.com>
*
* The software and information contained herein is proprietary and
* confidential to JingJiaMicro Electronics. This software can only be
* used by JingJiaMicro Electronics Corporation. Any use, reproduction,
* or disclosure without the written permission of JingJiaMicro
* Electronics Corporation is strictly prohibited.
*/
#include <linux/debugfs.h>
#include "mwv207_bo.h"
#include "mwv207_drm.h"

#ifdef CONFIG_DEBUG_FS

static int mwv207_debugfs_vram_size_show(struct seq_file *m, void *unused)
{
	struct mwv207_device *mdev = m->private;

	seq_printf(m, "%llu\n", mdev->hw.vram_size);

	return 0;
}
DEFINE_SHOW_ATTRIBUTE(mwv207_debugfs_vram_size);

static int mwv207_debugfs_gtt_size_show(struct seq_file *m, void *unused)
{
	struct mwv207_device *mdev = m->private;

	seq_printf(m, "%llu\n", mdev->gtt_size);

	return 0;
}
DEFINE_SHOW_ATTRIBUTE(mwv207_debugfs_gtt_size);

static int mwv207_debugfs_moved_bytes_show(struct seq_file *m, void *unused)
{
	struct mwv207_device *mdev = m->private;

	seq_printf(m, "%llu\n", atomic64_read(&mdev->moved_bytes));

	return 0;
}
DEFINE_SHOW_ATTRIBUTE(mwv207_debugfs_moved_bytes);

static int mwv207_debugfs_vram_used_show(struct seq_file *m, void *unused)
{
	struct mwv207_device *mdev = m->private;

	seq_printf(m, "%llu\n", atomic64_read(&mdev->vram_used_bytes));

	return 0;
}
DEFINE_SHOW_ATTRIBUTE(mwv207_debugfs_vram_used);

static int mwv207_debugfs_visible_vram_used_show(struct seq_file *m,
						  void *unused)
{
	struct mwv207_device *mdev = m->private;

	seq_printf(m, "%llu\n",
		   atomic64_read(&mdev->visible_vram_used_bytes));

	return 0;
}
DEFINE_SHOW_ATTRIBUTE(mwv207_debugfs_visible_vram_used);

static int mwv207_debugfs_gtt_used_show(struct seq_file *m, void *unused)
{
	struct mwv207_device *mdev = m->private;

	seq_printf(m, "%llu\n", atomic64_read(&mdev->gtt_used_bytes));

	return 0;
}
DEFINE_SHOW_ATTRIBUTE(mwv207_debugfs_gtt_used);

static int mwv207_debugfs_vram_pinned_show(struct seq_file *m, void *unused)
{
	struct mwv207_device *mdev = m->private;

	seq_printf(m, "%llu\n", mdev->vram_pinned_bytes);

	return 0;
}
DEFINE_SHOW_ATTRIBUTE(mwv207_debugfs_vram_pinned);

static int mwv207_debugfs_gtt_pinned_show(struct seq_file *m, void *unused)
{
	struct mwv207_device *mdev = m->private;

	seq_printf(m, "%llu\n", mdev->gtt_pinned_bytes);

	return 0;
}
DEFINE_SHOW_ATTRIBUTE(mwv207_debugfs_gtt_pinned);

static int mwv207_debugfs_3d_clusters_show(struct seq_file *m, void *unused)
{
	struct mwv207_device *mdev = m->private;

	seq_printf(m, "%d\n", mdev->hw.nr_3d_cluster);

	return 0;
}
DEFINE_SHOW_ATTRIBUTE(mwv207_debugfs_3d_clusters);

static int mwv207_debugfs_2d_clusters_show(struct seq_file *m, void *unused)
{
	struct mwv207_device *mdev = m->private;

	seq_printf(m, "%d\n", mdev->hw.nr_2d_cluster);

	return 0;
}
DEFINE_SHOW_ATTRIBUTE(mwv207_debugfs_2d_clusters);

static int mwv207_debugfs_fus_clusters_show(struct seq_file *m, void *unused)
{
	struct mwv207_device *mdev = m->private;

	seq_printf(m, "%d\n", mdev->hw.nr_fus_cluster);

	return 0;
}
DEFINE_SHOW_ATTRIBUTE(mwv207_debugfs_fus_clusters);

static int mwv207_debugfs_num_dma_show(struct seq_file *m, void *unused)
{
	struct mwv207_device *mdev = m->private;

	seq_printf(m, "%d\n", mdev->hw.nr_pipe[0x4]);

	return 0;
}
DEFINE_SHOW_ATTRIBUTE(mwv207_debugfs_num_dma);

static int mwv207_debugfs_num_dec_show(struct seq_file *m, void *unused)
{
	struct mwv207_device *mdev = m->private;

	seq_printf(m, "%d\n", mdev->hw.nr_pipe[0x1]);

	return 0;
}
DEFINE_SHOW_ATTRIBUTE(mwv207_debugfs_num_dec);

static int mwv207_debugfs_num_enc_show(struct seq_file *m, void *unused)
{
	struct mwv207_device *mdev = m->private;

	seq_printf(m, "%d\n", mdev->hw.nr_pipe[0x2]);

	return 0;
}
DEFINE_SHOW_ATTRIBUTE(mwv207_debugfs_num_enc);

int mwv207_debugfs_init(struct mwv207_device *mdev)
{
	struct dentry *root = mdev->base.primary->debugfs_root;
	struct dentry *ent;

	ent = debugfs_create_file("vram_size", 0444, root, mdev,
				  &mwv207_debugfs_vram_size_fops);
	if (IS_ERR(ent)) {
		DRM_ERROR("unable to create vram size debugsfs file\n");
		return PTR_ERR(ent);
	}

	ent = debugfs_create_file("gtt_size", 0444, root, mdev,
				  &mwv207_debugfs_gtt_size_fops);
	if (IS_ERR(ent)) {
		DRM_ERROR("unable to create gtt size debugsfs file\n");
		return PTR_ERR(ent);
	}

	ent = debugfs_create_file("moved_bytes", 0444, root, mdev,
				  &mwv207_debugfs_moved_bytes_fops);
	if (IS_ERR(ent)) {
		DRM_ERROR("unable to create moved bytes debugsfs file\n");
		return PTR_ERR(ent);
	}

	ent = debugfs_create_file("vram_used_bytes", 0444, root, mdev,
				  &mwv207_debugfs_vram_used_fops);
	if (IS_ERR(ent)) {
		DRM_ERROR("unable to create vram used bytes debugsfs file\n");
		return PTR_ERR(ent);
	}

	ent = debugfs_create_file("visible_vram_used_bytes", 0444, root, mdev,
				  &mwv207_debugfs_visible_vram_used_fops);
	if (IS_ERR(ent)) {
		DRM_ERROR("unable to create visible vram used debugsfs file\n");
		return PTR_ERR(ent);
	}

	ent = debugfs_create_file("gtt_used_bytes", 0444, root, mdev,
				  &mwv207_debugfs_gtt_used_fops);
	if (IS_ERR(ent)) {
		DRM_ERROR("unable to create gtt used bytes debugsfs file\n");
		return PTR_ERR(ent);
	}

	ent = debugfs_create_file("vram_pinned_bytes", 0444, root, mdev,
				  &mwv207_debugfs_vram_pinned_fops);
	if (IS_ERR(ent)) {
		DRM_ERROR("unable to create vram pinned bytes debugsfs file\n");
		return PTR_ERR(ent);
	}

	ent = debugfs_create_file("gtt_pinned_bytes", 0444, root, mdev,
				  &mwv207_debugfs_gtt_pinned_fops);
	if (IS_ERR(ent)) {
		DRM_ERROR("unable to create gtt pinned bytes debugsfs file\n");
		return PTR_ERR(ent);
	}

	ent = debugfs_create_file("3d_clusters", 0444, root, mdev,
				  &mwv207_debugfs_3d_clusters_fops);
	if (IS_ERR(ent)) {
		DRM_ERROR("unable to create 3d clusters debugsfs file\n");
		return PTR_ERR(ent);
	}

	ent = debugfs_create_file("2d_clusters", 0444, root, mdev,
				  &mwv207_debugfs_2d_clusters_fops);
	if (IS_ERR(ent)) {
		DRM_ERROR("unable to create 2d clusters debugsfs file\n");
		return PTR_ERR(ent);
	}

	ent = debugfs_create_file("fus_clusters", 0444, root, mdev,
				  &mwv207_debugfs_fus_clusters_fops);
	if (IS_ERR(ent)) {
		DRM_ERROR("unable to create fus clusters debugsfs file\n");
		return PTR_ERR(ent);
	}

	ent = debugfs_create_file("num_dma", 0444, root, mdev,
				  &mwv207_debugfs_num_dma_fops);
	if (IS_ERR(ent)) {
		DRM_ERROR("unable to create num dma debugsfs file\n");
		return PTR_ERR(ent);
	}

	ent = debugfs_create_file("num_dec", 0444, root, mdev,
				  &mwv207_debugfs_num_dec_fops);
	if (IS_ERR(ent)) {
		DRM_ERROR("unable to create num dec debugsfs file\n");
		return PTR_ERR(ent);
	}

	ent = debugfs_create_file("num_enc", 0444, root, mdev,
				  &mwv207_debugfs_num_enc_fops);
	if (IS_ERR(ent)) {
		DRM_ERROR("unable to create num enc debugsfs file\n");
		return PTR_ERR(ent);
	}

	return 0;
}

#endif
