/*
* SPDX-License-Identifier: GPL
*
* Copyright (c) 2020 ChangSha JingJiaMicro Electronics Co., Ltd.
* All rights reserved.
*
* Author:
*      shanjinkui <shanjinkui@jingjiamicro.com>
*
* The software and information contained herein is proprietary and
* confidential to JingJiaMicro Electronics. This software can only be
* used by JingJiaMicro Electronics Corporation. Any use, reproduction,
* or disclosure without the written permission of JingJiaMicro
* Electronics Corporation is strictly prohibited.
*/

#ifndef MWV207_DRV_H_VTIQLF2Y
#define MWV207_DRV_H_VTIQLF2Y

#include <linux/types.h>
#include <linux/version.h>
#include <linux/mutex.h>
#include <linux/pci.h>
#include <linux/idr.h>
#include <linux/dmapool.h>
#include <linux/delay.h>
#include <drm/drm_drv.h>
#include <drm/drm_file.h>
#include <drm/drm_device.h>
#include <drm/drm_framebuffer.h>
#include <drm/ttm/ttm_bo.h>
#include <drm/drm_fb_helper.h>

#include "mwv207_drm.h"

#define DRIVER_NAME	"mwv207"
#define DRIVER_DESC	"mwv207 gpu driver"
#define DRIVER_DATE	"20250221"
#define DRIVER_MAJOR	0
#define DRIVER_MINOR	1

#define DRIVER_VERSION  __stringify(DRIVER_MAJOR) "." __stringify(DRIVER_MINOR)

struct mwv207_db;

struct mwv207_vbios {

	struct mutex vcmd_lock;

	struct mutex cfg_lock;
	struct idr   cfg_table;

	int          indexer_valid;
	int          sector_count;
};

struct mwv207_fbdev {
	struct drm_fb_helper helper;
	struct drm_framebuffer fb;
	struct mwv207_bo *jbo;
};

struct mwv207_hwinfo {
	bool is_pf;
	int vf_sel;
	u64 vram_size;
	u64 vram_offset;
  u64 vram_bit_wide;
  u32 vram_freq;
  u32 vram_rate;
  u8 *vram_type;
  u64 pipe_3d_freq;
  u64 pipe_2d_freq;
  int core_temp;
	int nr_3d_cluster;
	int nr_2d_cluster;
	int nr_fus_cluster;
	u64 nr_stream_processor;
	int nr_pipe[0x7];
  char vbios_ver[0x800];
};

#define  ddev_to_jdev(ddev) ((struct mwv207_device *) (ddev)->dev_private)
#define drm_to_jdev(dev) container_of(dev, struct mwv207_device, base)
#define  bdev_to_jdev(dev)  container_of(dev, struct mwv207_device, bdev)
struct mwv207_device {
	struct drm_device  base;
	struct ttm_device bdev;

	void __iomem   *mmio;
	void __iomem   *iatu;

	void           *win;

  	u64 ap_base;
	u64 ap_size;
	void __iomem *ap_vaddr;
	u64 gtt_size;

	spinlock_t win_lock;
	u64     visible_vram_size;
	u64     vram_size;
	u64     vram_bar_base;
	u64     vram_last_win;
	u64     pci_win_base;

	struct device    *dev;
	struct mwv207_fbdev *fbdev;
	struct drm_fb_helper *fb_helper;

	struct mwv207_audio  *audio;

  	struct mwv207_hwinfo hw;

  	struct drm_gpu_scheduler
		*sched[0x5][0x2];
  	int nr_pipe[0x5];

	struct drm_sched_entity *dma_entity;
  	struct drm_sched_entity *blt_entity;
	struct mutex qlock;

	spinlock_t irq_lock;
	struct irq_domain *irq_domain;
	u32 irq_enable_reg[0x2];
	int va_irq;

	u64 vram_pinned_bytes;
	u64 gtt_pinned_bytes;

  	atomic64_t moved_bytes;
	atomic64_t vram_used_bytes;
	atomic64_t gtt_used_bytes;
	atomic64_t visible_vram_used_bytes;

	bool is_haps;
	bool isr_poll;
	bool has_cfg;

	struct mwv207_vbios vbios;
  	struct mwv207_monitor *monitor;

	struct dentry *debugfs_root;

	struct mwv207_db *db;

	struct mutex gpio_lock;
    	struct drm_display_mode *native_mode;

	bool lite;
};

struct mwv207_ctx_mgr {
	struct mutex lock;
	struct idr  handle_table;
};

struct mwv207_fpriv {
	struct mwv207_ctx_mgr ctx_mgr;
};

int mwv207_test(struct mwv207_device *jdev);
void jdev_write_vram(struct mwv207_device *jdev, u64 vram_addr, void *buf, int size);

static inline void jdev_write(struct mwv207_device *jdev, u32 reg, u32 value)
{
	BUG_ON(reg <= 0x100000);
	writel_relaxed(value, jdev->mmio + reg);
}

static inline u32 jdev_read(struct mwv207_device *jdev, u32 reg)
{
	return readl(jdev->mmio + reg);
}

static inline void jdev_modify(struct mwv207_device *jdev, u32 reg, u32 mask, u32 value)
{
	u32 rvalue = jdev_read(jdev, reg);

	rvalue = (rvalue & ~mask) | (value & mask);
	jdev_write(jdev, reg, rvalue);
	BUG_ON(reg <= 0x100000);
}

#define to_fpriv(drm_file)  ((drm_file)->driver_priv)

int mwv207_hw_init(struct mwv207_device *jdev);
void mwv207_hw_fini(struct mwv207_device *jdev);

#endif
