/*
* SPDX-License-Identifier: GPL
*
* Copyright (c) 2020 ChangSha JingJiaMicro Electronics Co., Ltd.
* All rights reserved.
*
* Author:
*      shanjinkui <shanjinkui@jingjiamicro.com>
*
* The software and information contained herein is proprietary and
* confidential to JingJiaMicro Electronics. This software can only be
* used by JingJiaMicro Electronics Corporation. Any use, reproduction,
* or disclosure without the written permission of JingJiaMicro
* Electronics Corporation is strictly prohibited.
*/
#include "mwv207_drv.h"
#include "mwv207_info.h"
#include "mwv207_debugfs.h"
#include "mwv207_sched.h"
#include "mwv207_vbios.h"

static ssize_t temperature_show(struct device *dev,
				    struct device_attribute *attr,
				    char *buf)
{

  struct mwv207_device *jdev = dev_get_drvdata(dev);
  s16 temp = jdev_read(jdev,0x9B0150);
	return sprintf(buf, "%u\n", (u32)temp);
}
static DEVICE_ATTR_RO(temperature);

static ssize_t vram_type_show(struct device *dev,
				    struct device_attribute *attr,
				    char *buf)
{
	struct mwv207_device *jdev = dev_get_drvdata(dev);
  return sprintf(buf, "%s\n", jdev->hw.vram_type);
}
static DEVICE_ATTR_RO(vram_type);

static ssize_t nr_stream_show(struct device *dev,
				    struct device_attribute *attr,
				    char *buf)
{
	struct mwv207_device *jdev = dev_get_drvdata(dev);

	return sprintf(buf, "%llu\n", jdev->hw.nr_stream_processor);
}
static DEVICE_ATTR_RO(nr_stream);

static ssize_t vram_rate_show(struct device *dev,
				    struct device_attribute *attr,
				    char *buf)
{
	struct mwv207_device *jdev = dev_get_drvdata(dev);

  u32 freq =0;
  u32 rate =0;

  mwv207_vbios_get_vram_pll(jdev,&freq,&rate);
  jdev->hw.vram_rate = rate;
  jdev->hw.vram_freq = freq;

	return sprintf(buf, "%u\n", jdev->hw.vram_rate);
}
static DEVICE_ATTR_RO(vram_rate);

static ssize_t vram_freq_show(struct device *dev,
				    struct device_attribute *attr,
				    char *buf)
{
	struct mwv207_device *jdev = dev_get_drvdata(dev);

  u32 freq =0;
  u32 rate =0;

  mwv207_vbios_get_vram_pll(jdev,&freq,&rate);
  jdev->hw.vram_rate = rate;
  jdev->hw.vram_freq = freq;

	return sprintf(buf, "%u\n", jdev->hw.vram_freq);
}
static DEVICE_ATTR_RO(vram_freq);

static ssize_t vram_bit_wide_show(struct device *dev,
				    struct device_attribute *attr,
				    char *buf)
{
	struct mwv207_device *jdev = dev_get_drvdata(dev);

	return sprintf(buf, "%llu\n", jdev->hw.vram_bit_wide);
}
static DEVICE_ATTR_RO(vram_bit_wide);

static ssize_t fb_memory_total_show(struct device *dev,
				    struct device_attribute *attr,
				    char *buf)
{
	struct mwv207_device *jdev = dev_get_drvdata(dev);

	return sprintf(buf, "%llu\n", jdev->hw.vram_size);
}
static DEVICE_ATTR_RO(fb_memory_total);

static ssize_t fb_memory_used_show(struct device *dev,
				   struct device_attribute *attr,
				   char *buf)
{
	struct mwv207_device *jdev = dev_get_drvdata(dev);

	return sprintf(buf, "%llu\n", atomic64_read(&jdev->vram_used_bytes));
}
static DEVICE_ATTR_RO(fb_memory_used);

static ssize_t bar_memory_total_show(struct device *dev,
				     struct device_attribute *attr,
				     char *buf)
{
	struct mwv207_device *jdev = dev_get_drvdata(dev);

	return sprintf(buf, "%llu\n", jdev->visible_vram_size);
}
static DEVICE_ATTR_RO(bar_memory_total);

static ssize_t bar_memory_used_show(struct device *dev,
				    struct device_attribute *attr,
				    char *buf)
{
	struct mwv207_device *jdev = dev_get_drvdata(dev);

	return sprintf(buf, "%llu\n",
		       atomic64_read(&jdev->visible_vram_used_bytes));
}
static DEVICE_ATTR_RO(bar_memory_used);

static ssize_t gtt_memory_total_show(struct device *dev,
				     struct device_attribute *attr,
				     char *buf)
{
	struct mwv207_device *jdev = dev_get_drvdata(dev);

	return sprintf(buf, "%llu\n", jdev->gtt_size);
}
static DEVICE_ATTR_RO(gtt_memory_total);

static ssize_t gtt_memory_used_show(struct device *dev,
				    struct device_attribute *attr,
				    char *buf)
{
	struct mwv207_device *jdev = dev_get_drvdata(dev);

	return sprintf(buf, "%llu\n", atomic64_read(&jdev->gtt_used_bytes));
}
static DEVICE_ATTR_RO(gtt_memory_used);

static ssize_t fb_memory_pinned_show(struct device *dev,
				     struct device_attribute *attr,
				     char *buf)
{
	struct mwv207_device *jdev = dev_get_drvdata(dev);

	return sprintf(buf, "%llu\n", jdev->vram_pinned_bytes);
}
static DEVICE_ATTR_RO(fb_memory_pinned);

static ssize_t gtt_memory_pinned_show(struct device *dev,
				      struct device_attribute *attr,
				      char *buf)
{
	struct mwv207_device *jdev = dev_get_drvdata(dev);

	return sprintf(buf, "%llu\n", jdev->gtt_pinned_bytes);
}
static DEVICE_ATTR_RO(gtt_memory_pinned);

static ssize_t memory_moved_show(struct device *dev,
				 struct device_attribute *attr,
				 char *buf)
{
	struct mwv207_device *jdev = dev_get_drvdata(dev);

	return sprintf(buf, "%llu\n", atomic64_read(&jdev->moved_bytes));
}
static DEVICE_ATTR_RO(memory_moved);

static ssize_t usage_2d_show(struct device *dev,
			     struct device_attribute *attr,
			     char *buf)
{
	struct mwv207_device *jdev = dev_get_drvdata(dev);
	u32 usage = mwv207_pipe_get_usage(jdev, 0x3, 0);

	return sprintf(buf, "%d\n", usage);
}
static DEVICE_ATTR_RO(usage_2d);

static ssize_t usage_3d_show(struct device *dev,
			     struct device_attribute *attr,
			     char *buf)
{
	struct mwv207_device *jdev = dev_get_drvdata(dev);
	u32 usage = mwv207_pipe_get_usage(jdev, 0x0, 0);

	return sprintf(buf, "%d\n", usage);
}
static DEVICE_ATTR_RO(usage_3d);

static ssize_t usage_enc0_show(struct device *dev,
			       struct device_attribute *attr,
			       char *buf)
{
	struct mwv207_device *jdev = dev_get_drvdata(dev);
	u32 usage = mwv207_pipe_get_usage(jdev, 0x2, 0);

	return sprintf(buf, "%d\n", usage);
}
static DEVICE_ATTR_RO(usage_enc0);

static ssize_t usage_dec0_show(struct device *dev,
			       struct device_attribute *attr,
			       char *buf)
{
	struct mwv207_device *jdev = dev_get_drvdata(dev);
	u32 usage = mwv207_pipe_get_usage(jdev, 0x1, 0);

	return sprintf(buf, "%d\n", usage);
}
static DEVICE_ATTR_RO(usage_dec0);

static ssize_t usage_enc1_show(struct device *dev,
			       struct device_attribute *attr,
			       char *buf)
{
	struct mwv207_device *jdev = dev_get_drvdata(dev);
	u32 usage = mwv207_pipe_get_usage(jdev, 0x2, 1);

	return sprintf(buf, "%d\n", usage);
}
static DEVICE_ATTR_RO(usage_enc1);

static ssize_t usage_dec1_show(struct device *dev,
			       struct device_attribute *attr,
			       char *buf)
{
	struct mwv207_device *jdev = dev_get_drvdata(dev);
	u32 usage = mwv207_pipe_get_usage(jdev, 0x1, 1);

	return sprintf(buf, "%d\n", usage);
}
static DEVICE_ATTR_RO(usage_dec1);

static ssize_t vbios_version_show(struct device *dev,
				  struct device_attribute *attr,
				  char *buf)
{
	struct mwv207_device *jdev = dev_get_drvdata(dev);
  const struct mwv207_vdat *dat;
  const char *ver;
  dat = mwv207_vbios_vdat(jdev, 0x1FF0);
  ver = (const char *)dat->dat;

	return sprintf(buf, "%s\n", ver? ver: "Unknown");
}
static DEVICE_ATTR_RO(vbios_version);

static ssize_t fan_speed_show(struct device *dev,
			      struct device_attribute *attr,
			      char *buf)
{

	return sprintf(buf, "%u\n", 0x1234u);
}
static DEVICE_ATTR_RO(fan_speed);

static ssize_t power_show(struct device *dev,
			  struct device_attribute *attr,
			  char *buf)
{

	return sprintf(buf, "%u\n", 0xffffffffu);
}
static DEVICE_ATTR_RO(power);

static ssize_t voltage_show(struct device *dev,
			    struct device_attribute *attr,
			    char *buf)
{
  struct mwv207_device *jdev = dev_get_drvdata(dev);
  u32 volt= jdev_read(jdev,0x9B0154);

	return sprintf(buf, "%u mv\n", volt);
}
static DEVICE_ATTR_RO(voltage);

static struct attribute *mwv207_attrs[] = {
	&dev_attr_vram_type.attr,
	&dev_attr_vram_bit_wide.attr,
	&dev_attr_vram_freq.attr,
	&dev_attr_vram_rate.attr,
	&dev_attr_fb_memory_total.attr,
	&dev_attr_fb_memory_used.attr,
	&dev_attr_nr_stream.attr,
	&dev_attr_temperature.attr,
	&dev_attr_bar_memory_total.attr,
	&dev_attr_bar_memory_used.attr,
	&dev_attr_usage_2d.attr,
	&dev_attr_usage_3d.attr,
	&dev_attr_usage_enc0.attr,
	&dev_attr_usage_dec0.attr,
	&dev_attr_usage_enc1.attr,
	&dev_attr_usage_dec1.attr,
	&dev_attr_vbios_version.attr,
	&dev_attr_fan_speed.attr,
	&dev_attr_power.attr,
	&dev_attr_voltage.attr,
	&dev_attr_gtt_memory_total.attr,
	&dev_attr_gtt_memory_used.attr,
	&dev_attr_fb_memory_pinned.attr,
	&dev_attr_gtt_memory_pinned.attr,
	&dev_attr_memory_moved.attr,
	NULL,
};

static const struct attribute_group mwv207_group = {
	.name  = "mwv207_hwmon",
	.attrs = mwv207_attrs,
};

static const struct attribute_group *mwv207_groups[] = {
	&mwv207_group,
	NULL,
};

int mwv207_info_init(struct mwv207_device *jdev)
{
	int ret;

	ret = sysfs_create_groups(&jdev->dev->kobj, mwv207_groups);
	if (ret) {
		dev_err(jdev->dev, "failed to create sysfs groups");
		return ret;
	}

	ret = mwv207_debugfs_init(jdev);
	if (ret)
		dev_info(jdev->dev, "create debugfs files failed(%d)", ret);

	return 0;
}

void mwv207_info_fini(struct mwv207_device *jdev)
{
	sysfs_remove_groups(&jdev->dev->kobj, mwv207_groups);
}
